/*
 * video_in_main.c
 *
 *  Created on: Apr 18, 2017
 *      Author: fb
 */

#include <stdint.h>
#include "system.h"
#include "altera_up_avalon_video_pixel_buffer_dma.h"
#include "altera_up_avalon_video_character_buffer_with_dma.h"
#include "altera_up_avalon_parallel_port.h"
#include "../Inc/video_in_dma.h"

#include "../Inc/config.h"


int main()
{
	uint32_t ledg = 0;
	uint32_t key_status;

	enum {LIVE, PAUSED} camera_state = LIVE;

	alt_up_pixel_buffer_dma_dev* pixel_buffer_dma_dev_ptr;
	alt_up_char_buffer_dev* char_buffer_dev_ptr;

	alt_up_parallel_port_dev* green_leds_dev_ptr;
	alt_up_parallel_port_dev* pusbuttons_dev_ptr;

	/* open devices */
	char_buffer_dev_ptr = alt_up_char_buffer_open_dev("/dev/Video_character_buffer");
	pixel_buffer_dma_dev_ptr = alt_up_pixel_buffer_dma_open_dev(PIXEL_BUFFER_DMA_NAME);
	green_leds_dev_ptr = alt_up_parallel_port_open_dev(GREEN_LEDS_NAME);
	pusbuttons_dev_ptr = alt_up_parallel_port_open_dev(PUSH_BUTTONS_NAME);

	/* set back buffer address */
	video_in_dma_change_back_buffer_address(VIDEO_DMA_BASE, BACK_BUFFER_ADDRESS);
	ledg = BIT(LED_ON) | BIT(LED_PLAY);
	alt_up_parallel_port_write_data(green_leds_dev_ptr, ledg);
	alt_up_char_buffer_clear(char_buffer_dev_ptr);
	alt_up_char_buffer_string(char_buffer_dev_ptr, STRING_PLAY, STRING_X_POS, STRING_Y_POS);

	while(1)
	{
		/* read push buttons activity */
		key_status = alt_up_parallel_port_read_edge_capture(pusbuttons_dev_ptr);
		alt_up_parallel_port_clear_edge_capture(pusbuttons_dev_ptr);
		ledg = (ledg & 0xF1) | alt_up_parallel_port_read_data(pusbuttons_dev_ptr);

		switch(camera_state)
		{
			case LIVE:
				if(key_status & BIT(KEY_PAUSE))
				{
					camera_state = PAUSED;
					video_in_dma_swap_buffers(VIDEO_DMA_BASE);
					ledg = (ledg & ~BIT(LED_PLAY)) | BIT(LED_PAUSED);
					alt_up_char_buffer_clear(char_buffer_dev_ptr);
					alt_up_char_buffer_string(char_buffer_dev_ptr, STRING_PAUSE, STRING_X_POS, STRING_Y_POS);
					/* wait for the buffer swap to complete */
					while(video_in_dma_check_swap_buffers_status(VIDEO_DMA_BASE)) { ; }
				}
			break;

			case PAUSED:
				if(key_status & BIT(KEY_PLAY))
				{
					camera_state = LIVE;
					video_in_dma_swap_buffers(VIDEO_DMA_BASE);
					ledg = (ledg & ~BIT(LED_PAUSED)) | BIT(LED_PLAY);
					alt_up_char_buffer_clear(char_buffer_dev_ptr);
					alt_up_char_buffer_string(char_buffer_dev_ptr, STRING_PLAY, STRING_X_POS, STRING_Y_POS);
					/* wait for the buffer swap to complete */
					while(video_in_dma_check_swap_buffers_status(VIDEO_DMA_BASE)) { ; }
				}
			break;

			default:
			break;
		}
		alt_up_parallel_port_write_data(green_leds_dev_ptr, ledg);
		key_status = 0;
	}


}
